import tkinter as tk
from tkinter import simpledialog, messagebox
import time
import base64
import hmac
import hashlib
import os

# פונקציות עזר
def save_token(token):
    with open("token.txt", "w") as file:
        file.write(token)

def load_token():
    if os.path.exists("token.txt"):
        with open("token.txt", "r") as file:
            return file.read()
    return None

def generate_totp(secret):
    try:
        secret = base64.b32decode(secret.upper())
        time_counter = int(time.time() // 30)
        msg = time_counter.to_bytes(8, 'big')
        hash = hmac.new(secret, msg, hashlib.sha1).digest()
        offset = hash[-1] & 0x0F
        code = (int.from_bytes(hash[offset:offset+4], 'big') & 0x7FFFFFFF) % 1000000
        return str(code).zfill(6)
    except Exception as e:
        raise ValueError("טוקן לא תקין")

def update_code():
    global token
    if token:
        try:
            code = generate_totp(token)
            countdown = 30 - int(time.time() % 30)
            lbl_code.config(text=f"סיסמה זמנית: {code}\n\nזמן נותר: {countdown} שניות")
            btn_copy_code.config(state=tk.NORMAL)  # הפעלת כפתור העתקה
        except ValueError:
            lbl_code.config(text="שגיאה: טוקן לא תקין")
            btn_copy_code.config(state=tk.DISABLED)
    else:
        lbl_code.config(text="אין טוקן מוגדר")
        btn_copy_code.config(state=tk.DISABLED)
    lbl_code.after(1000, update_code)

def set_new_token():
    global token
    new_token = token_entry.get().strip()
    if new_token:
        try:
            # בדיקה אם הטוקן תקין
            generate_totp(new_token)
            token = new_token
            save_token(token)
            messagebox.showinfo("טוקן נשמר", "הטוקן החדש נשמר בהצלחה!")
            token_entry.delete(0, tk.END)  # מחיקת התוכן מתיבת הטקסט
            token_entry.insert(0, token)
        except ValueError:
            messagebox.showerror("שגיאה", "הטוקן שהוזן אינו תקין. נסה שוב.")
            token_entry.delete(0, tk.END)  # מחיקת הטוקן השגוי
    update_code()

def copy_code():
    global token
    if token:
        try:
            code = generate_totp(token)
            root.clipboard_clear()
            root.clipboard_append(code)
            root.update()
            messagebox.showinfo("העתקה", "הקוד הועתק ללוח!")
        except ValueError:
            messagebox.showerror("שגיאה", "לא ניתן להעתיק קוד לטוקן שגוי.")

# ממשק משתמש
root = tk.Tk()
root.title("מחולל קודים זמניים")
root.geometry("350x300")
root.configure(bg="#f0f0f0")

lbl_code = tk.Label(root, text="", font=("Arial", 14), bg="#f0f0f0", fg="#333")
lbl_code.pack(pady=20)

frame = tk.Frame(root, bg="#f0f0f0")
frame.pack(pady=10)

token_label = tk.Label(frame, text="הכנס טוקן:", bg="#f0f0f0", font=("Arial", 12))
token_label.pack(side=tk.LEFT, padx=5)

token_entry = tk.Entry(frame, font=("Arial", 12), width=20)
token_entry.pack(side=tk.LEFT)

btn_set_token = tk.Button(root, text="שמור טוקן", command=set_new_token, bg="#007ACC", fg="white", font=("Arial", 12))
btn_set_token.pack(pady=10)

btn_copy_code = tk.Button(root, text="העתק קוד", command=copy_code, bg="#28a745", fg="white", font=("Arial", 12), state=tk.DISABLED)
btn_copy_code.pack(pady=10)

btn_exit = tk.Button(root, text="יציאה", command=root.destroy, bg="#D9534F", fg="white", font=("Arial", 12))
btn_exit.pack(pady=10)

# טעינת טוקן קיים והתחלת התהליך
token = load_token()
if token:
    token_entry.insert(0, token)  # הצגת הטוקן בתיבת הטקסט
update_code()

root.mainloop()
